// -*- c++ -*-
/*
 *  MICO --- an Open Source CORBA implementation
 *  Copyright (c) 1997-2001 by The Mico Team
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  For more information, visit the MICO Home Page at
 *  http://www.mico.org/
 */

#ifndef __corba_h__
#define __corba_h__


/***************************** C headers ****************************/

#include <mico/assert.h>
#include <stddef.h> // for wchar_t
#ifndef _POCKET_PC
#include <sys/types.h>
#include <errno.h>
#endif

/***************************** config *******************************/

// For switching on debug code uncomment line below and comment line under it
//#define MTDEBUG
#undef MTDEBUG

#if defined(_WIN32) && !defined(__MINGW32__)
#if defined(_MSC_VER)  && (_MSC_VER < 1300)
#error You need at least VC 7.0 (MSDEV.NET) to compile MICO successfully
#endif
#include <mico/config-win32.h>
#elif defined (_POCKET_PC)
#include <mico/config-ppc.h>
#else
#include <mico/config.h>
#endif


#ifndef TRUE
#define TRUE 1
#endif

#ifndef FALSE
#define FALSE 0
#endif


#if (defined(PIC) && defined(HAVE_SHARED_EXCEPTS)) || \
    (!defined(PIC) && defined(HAVE_EXCEPTS))
#define HAVE_EXCEPTIONS 1
#endif

#ifdef MICO_CONF_NO_EXCEPTIONS
#undef HAVE_EXCEPTS
#undef HAVE_SHARED_EXCEPTS
#undef HAVE_EXCEPTIONS
#endif


#ifdef HAVE_NAMESPACE
#define MICO_NAMESPACE_DECL namespace
#define MICO_NAMESPACE_END_DECL 
#define MICO_EXPORT_DECL extern
#define MICO_EXPORT_FCT_DECL extern
#define MICO_INLINE_FCT_DECL static inline
#define MICO_EXPORT_VAR_DECL extern
#define MICO_EXPORT_TYPEVAR_DECL extern
#else
#error "no namespaces"
#endif

#define MICO_EXPORT_CONST_DECL MICO_EXPORT_VAR_DECL


// kcg: following hack is not necessary for VC++ 7 and higher
// I've commented it out and it'll be removed in the future
//
// This Macro is used in the idl compiler even on unix
//#define _VC_NAMESPACE_HACK "_VCHACK__"

// kcg: following hack is not necessary for VC++ 7 and higher
// I've commented it out and it'll be removed in the future
//
// Assume that Namespace handling is buggy in Visual-C++
//#define _VC_NAMESPACE_BUG

#if defined (_WIN32) || defined (_POCKET_PC)
#undef environ

#ifdef BUILD_MICO_DLL
#undef  MICO_EXPORT_VAR_DECL
#undef  MICO_EXPORT_CONST_DECL
#define MICO_EXPORT_VAR_DECL extern
#define MICO_EXPORT_CONST_DECL extern
#define MICO_EXPORT_TYPEVAR_DECL extern
#else
#undef  MICO_EXPORT_VAR_DECL
#undef  MICO_EXPORT_CONST_DECL
#define MICO_EXPORT_VAR_DECL extern
#define MICO_EXPORT_CONST_DECL extern
#define MICO_EXPORT_TYPEVAR_DECL extern
#endif
#endif

#ifdef USE_MEMTRACE
// this enables memtracing for STL objects
// The malloc and free macros are undefined under STL includes
#include <mico/memtrace.h>
#endif

/***************************** STL headers **************************/


#ifdef HAVE_MINI_STL
#include <ministl/string>
#include <ministl/vector>
#include <ministl/map>
#include <ministl/set>
#include <ministl/list>

#else

#include <string>
#include <vector>
#include <map>
#include <set>
#include <list>
#include <queue>

#if defined(_WIN32) && !defined(__MINGW32__)
#include <limits>
#endif

#endif


typedef std::vector<int>::size_type mico_vec_size_type;

#ifdef USE_MEMTRACE
#undef malloc
#undef free
#endif
/************************** MICO templates **************************/

#include <mico/version.h>
#include <mico/types.h>
#include <mico/sequence.h>
#include <mico/sequence_special.h>
#include <mico/array.h>
#include <mico/template.h>
#include <mico/fixed.h>
#include <mico/native.h>

// MICOMT needs MICO::Logger from utils
//#include <mico/util.h>

/*************************** module MICOMT **************************/
// we already need to include os-thread even for ST build
#include <mico/os-thread.h>

/*
 * VC++ insanity
 */

#if defined(_WIN32) && !defined(__MINGW32__)
namespace CORBA {
	using namespace CORBA;
}
#ifndef MICO_CONF_NO_POA
namespace PortableServer {
	using namespace PortableServer;
}
#endif
#endif

/*************************** module CORBA ***************************/

#include <mico/basic.h>
#include <mico/magic.h>
#include <mico/address.h>
#include <mico/string.h>
#include <mico/tcconst.h>
#include <mico/ioptypes.h>
#include <mico/ior.h>
#include <mico/transport.h>
#include <mico/buffer.h>
#include <mico/codeset.h>
#include <mico/string.h>
#include <mico/codec.h>
#include <mico/except.h>
#include <mico/orb_excepts.h>
#include <mico/any.h>
#include <mico/object.h>
#include <mico/value.h>
#include <mico/basic_seq.h>
#include <mico/orb_fwd.h>
namespace PortableInterceptor {
	class ClientRequestInfo;
	class ServerRequestInfo;
	typedef ClientRequestInfo *ClientRequestInfo_ptr;
	typedef ServerRequestInfo *ServerRequestInfo_ptr;
}
#include <mico/dii.h>
#include <mico/dsi.h>
#include <mico/static.h>

#include <mico/policy.h>
#include <mico/current.h>

#include <mico/boa.h>

#ifndef MICO_CONF_NO_POA
namespace PortableServer {
  class ServantBase;
}

#ifndef MICO_CONF_NO_INTERCEPT
#if (defined(USE_CSL2)) || (defined(USE_CSIV2))
#include <mico/security/csi_base.h>
#endif // USE_CSL2 or USE_CSIV2

#include <mico/timebase.h>

#ifdef USE_CSL2
#  include <mico/service_info.h>
#  include <mico/security/security.h>
#  include <mico/security/DomainManager.h>
#  include <mico/security/odm.h>
#endif /* USE_CSL2  */
#endif // MICO_CONF_NO_INTERCEPT

#include <mico/poa.h>
#include <mico/poa_base.h>
#include <mico/poa_stubs.h>
#endif // MICO_CONF_NO_POA

#include <mico/tckind.h>
#include <mico/ir_base.h>
#include <mico/typecode.h>
#include <mico/typecode_seq.h>
#include <mico/valuetype.h>

#ifndef MICO_CONF_NO_IR
#include <mico/ir.h>
#endif // MICO_CONF_NO_IR
#include <mico/service_info.h>
#include <mico/orb_mico.h>
#ifndef MICO_CONF_NO_IMR
#include <mico/imr.h>
#endif // MICO_CONF_NO_IMR
#include <mico/valuetype_impl.h>


//  #ifdef USE_CSL2

//  /********************** module TimeBase ***********************/
//  // namespace TimeBase {

//  #ifdef _WIN32
//  	using namespace TimeBase;
//  #endif

//  #define MICO_MODULE_TimeBase
//  //#include <mico/timebase.h>
//  #undef MICO_MODULE_TimeBase
//  // };
//  extern CORBA::StaticTypeInfo *_marshaller_TimeBase_UtcT;
//  extern CORBA::StaticTypeInfo *_marshaller_TimeBase_IntervalT;
//  /*
//  MICO_NAMESPACE_DECL CosTime {
//  #ifdef _WIN32
//  	using namespace CosTime;
//  #endif
//  #define MICO_MODULE_CosTime
//  #include <mico/CosTime.h>
//  #undef MICO_MODULE_CosTime
//  };

//  */
//  /********************** module Security ***********************/

//  //MICO_NAMESPACE_DECL Security {
//  #ifdef _WIN32
//  	using namespace Security;
//  #endif
//  #define MICO_MODULE_Security
//  #include <mico/security.h>
//  #undef MICO_MODULE_Security
//  //};

//  /********************** module SecurityDomain ***********************/
//  // MICO_NAMESPACE_DECL SecurityDomain {
//  #ifdef _WIN32
//  	using namespace SecurityDomain;
//  #endif
//  #define MICO_MODULE_SecurityDomain
//   #include <mico/DomainManager.h>
//  #undef MICO_MODULE_SecurityDomain
//  //};

//  /********************** module ObjecDomainMapping ***********************/
//  // MICO_NAMESPACE_DECL ObjectDomainMapping {
//  #ifdef _WIN32
//  	using namespace ObjectDomainMapping;
//  #endif
//  #define MICO_MODULE_ObjectDomainMapping
//  #include <mico/odm.h>
//  #undef MICO_MODULE_ObjectDomainMapping
//  // };

//  #endif /* USE_CSL2  */


/********************** Global ******************************************/

#ifndef MICO_CONF_NO_DYNANY
#include <mico/dynany.h>
#endif

#ifndef MICO_CONF_NO_INTERCEPT
#include <mico/intercept.h>
#include <mico/pi.h>
namespace PortableInterceptor {
    void register_orb_initializer (PortableInterceptor::ORBInitializer_ptr init);
}
#endif

#include <mico/operators.h>
#include <mico/policy2.h>

// used in skeletons
CORBA::ULong mico_string_hash (const char *, CORBA::ULong);

#include <mico/throw.h>
#include <mico/template_impl.h>

#ifndef MICO_CONF_NO_INTERCEPT
#ifdef USE_CSL2
// #include <mico/security.h>
#  include <mico/security/securitylevel1.h>
#  include <mico/security/securitylevel2.h>
#  include <mico/security/SecurityAdmin.h>
#  include <mico/security/DomainManager.h>
#  include <mico/security/odm.h>
//#endif
#endif /* USE_CSL2  */
#endif // MICO_CONF_NO_INTERCEPT

#ifndef MICO_CONF_NO_INTERCEPT
#ifdef USE_CSIV2
#include <mico/security/csi.h>
#include <mico/security/csiiop.h>
#include <mico/security/gssup.h>
#include <mico/security/csiv2.h>
//#include <mico/security/csiv2_impl.h>
#endif // USE_CSIV2
#endif // MICO_CONF_NO_INTERCEPT
#endif // __corba_h__
